package info.aspirecn.iov.sjjh.supplier10000040.action;

import brave.Tracer;
import info.aspirecn.iov.sjjh.commons.lang.ChannelTypeHandleResponseObject;
import info.aspirecn.iov.sjjh.commons.lang.Constant;
import info.aspirecn.iov.sjjh.supplier10000040.constant.ChannelConstant;
import info.aspirecn.iov.sjjh.supplier10000040.property.SupplierProperties;
import info.aspirecn.iov.sjjh.supplier10000040.service.ChannelService;
import info.aspirecn.iov.sjjh.supplier10000040.task.AsyncDelayTask;
import info.aspirecn.iov.sjjh.supplier10000040.util.EsOpeateUtils;
import info.aspirecn.rdc.aspirecloud.node.except.utils.ErrorUtils;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.tomcat.util.ExceptionUtils;
import org.apache.tomcat.util.codec.binary.Base64;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import java.io.UnsupportedEncodingException;
import java.util.concurrent.*;

/**
 * @author: xusonglin
 * @create: 2020/4/22 14:59
 * @version: V1.0
 **/
@Slf4j
@RestController
public class ChannelAction {
    /**
     * 日志参数中的错误参数名
     */
    private static final String LOG_ERROR_PARA = "x-responseCode";
    /**
     * 日志参数中的是否收费
     */
    private static final String LOG_FEE_PARA = "x-isCharge";

    @Autowired
    EsOpeateUtils esOpeateUtils;
    @Autowired
    ChannelService channelService;
    @Autowired
    private SupplierProperties supplierProperties;
    @Autowired
    AsyncDelayTask asyncDelayTask;

    @Qualifier("mainThreadPoolExecutor")
    @Autowired
    ThreadPoolExecutor threadPoolExecutor;

    @Value("${supplier.delayedTime}")
    private Long delayedTime;

    private static final String LOG_TREACEID = "x-b3-traceid";
    private static final String LOG_SPAN_ID = "x-b3-spanid";
    private static final String LOG_PARENT_SPAN_ID = "x-b3-parentspanid";
    private static final String LOG_REAL_IP = "X-Real-IP";
    private static final String LOG_CHANNEL_ID = "channelId";
    private static final String LOG_CUSTOME_BODY = "x-requestQuery";
    private static final String LOG_PRICE = "price";
    private static final String LOG_REQ_TYPE = "req_type";
    private static final String LOG_PRODUCT_ID = "productId";
    private static final String LOG_USER_ID = "userid";

    @Autowired
    private Tracer tracer;

    @ApiOperation(value = "个人不良信息查询-评分", notes = "")
    @PostMapping(value = "/getIllegalInfoScore.do")
    public ChannelTypeHandleResponseObject getIllegalInfoScore(
            @ApiParam(value = "通道ID") @RequestHeader(name = "channelId") String channelId,
            @ApiParam(value = "超时时间,单位：毫秒", example = "10000") @RequestParam(name = "outTime", required = true) int outTime,
            @ApiParam(value = "请求参数JSON串") @RequestParam(name = "customBody", required = true) String customBody) {
        log.info("个人不良信息查询-评分请求--->channelId:{}--->outTime:{},--->customBody:{}", channelId, outTime, customBody);
        //日志记录通道类型 - 同步
        ServletRequestAttributes servletRequestAttributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        HttpServletRequest request = servletRequestAttributes.getRequest();
        //记录日志信息09
        recordQueryRequestLog(request);
        ChannelTypeHandleResponseObject responseObject = new ChannelTypeHandleResponseObject();
        try {
            responseObject = channelService.validateIllegalInfo(request, customBody, outTime, "1", ChannelConstant.ILLEGAL_INFO_SCORE);
            //日志记录通道响应码
            if (responseObject.getCode() != 1) {
                request.setAttribute(Constant.ACCESSLOG_MG_RESPONSE_CODE_KEY, Constant.CHANNEL_LOG_ERROR_CODE);
            } else {
                log.info("ifJk:{}", request.getHeader(Constant.CUSTOMER_RETURN_TYPE_PARA));
                if (Constant.CUSTOMER_RETURN_JK.equals(request.getHeader(Constant.CUSTOMER_RETURN_TYPE_PARA))) {

                    String code = supplierProperties.getCode().get(responseObject.getResultCode());
                    log.info("resultCode:{},jkCode:{}", responseObject.getResultCode(), code);
                    if (code != null) {
                        responseObject.setResultCode(Integer.valueOf(code));
                    }
                }
                request.setAttribute(Constant.ACCESSLOG_MG_RESPONSE_CODE_KEY, Constant.CHANNEL_LOG_SUCCESS_CODE);
            }
            //日志记录通道查询参数
            if (customBody != null) {
                try {
                    request.setAttribute(Constant.CHANNEL_LOG_QUERY, Base64.encodeBase64String(customBody.getBytes("utf-8")));
                } catch (UnsupportedEncodingException e) {
                    ExceptionUtils.handleThrowable(e);
                    log.error("异常信息：{}", e);
                }
            }

            request.setAttribute(Constant.ACCESSLOG_MG_ISCHARGE_KEY, responseObject.getIsCharge());
            return responseObject;
        } catch (Exception ex) {
            log.error("message:{}", ex);
            return responseObject;
        }
    }

    @ApiOperation(value = "个人不良信息查询-返回上游结果", notes = "")
    @PostMapping(value = "/getIllegalInfoResult.do")
    public ChannelTypeHandleResponseObject getIllegalInfoResult(
            @ApiParam(value = "通道ID") @RequestHeader(name = "channelId") String channelId,
            @ApiParam(value = "超时时间,单位：毫秒", example = "10000") @RequestParam(name = "outTime", required = true) int outTime,
            @ApiParam(value = "请求参数JSON串") @RequestParam(name = "customBody", required = true) String customBody) {
        log.info("个人不良信息查询-返回上游结果请求--->channelId:{}--->outTime:{},--->customBody:{}", channelId, outTime, customBody);
        //日志记录通道类型 - 同步
        ServletRequestAttributes servletRequestAttributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        HttpServletRequest request = servletRequestAttributes.getRequest();
        //记录日志信息09
        recordQueryRequestLog(request);
        ChannelTypeHandleResponseObject responseObject = new ChannelTypeHandleResponseObject();
        try {
            responseObject = channelService.validateIllegalInfo(request, customBody, outTime, "1", ChannelConstant.ILLEGAL_INFO_RESULT);
            //日志记录通道响应码
            if (responseObject.getCode() != 1) {
                request.setAttribute(Constant.ACCESSLOG_MG_RESPONSE_CODE_KEY, Constant.CHANNEL_LOG_ERROR_CODE);
            } else {
                log.info("ifJk:{}", request.getHeader(Constant.CUSTOMER_RETURN_TYPE_PARA));
                if (Constant.CUSTOMER_RETURN_JK.equals(request.getHeader(Constant.CUSTOMER_RETURN_TYPE_PARA))) {

                    String code = supplierProperties.getCode().get(responseObject.getResultCode());
                    log.info("resultCode:{},jkCode:{}", responseObject.getResultCode(), code);
                    if (code != null) {
                        responseObject.setResultCode(Integer.valueOf(code));
                    }
                }
                request.setAttribute(Constant.ACCESSLOG_MG_RESPONSE_CODE_KEY, Constant.CHANNEL_LOG_SUCCESS_CODE);
            }
            //日志记录通道查询参数
            if (customBody != null) {
                try {
                    request.setAttribute(Constant.CHANNEL_LOG_QUERY, Base64.encodeBase64String(customBody.getBytes("utf-8")));
                } catch (UnsupportedEncodingException e) {
                    ExceptionUtils.handleThrowable(e);
                    log.error("异常信息：{}", e);
                }
            }

            request.setAttribute(Constant.ACCESSLOG_MG_ISCHARGE_KEY, responseObject.getIsCharge());
            return responseObject;
        } catch (Exception ex) {
            log.error("message:{}", ex);
            return responseObject;
        }
    }

    @ApiOperation(value = "个人不良信息查询-简版", notes = "")
    @PostMapping(value = "/getIllegalInfoSimple.do")
    public ChannelTypeHandleResponseObject getIllegalInfoSimple(
            @ApiParam(value = "通道ID") @RequestHeader(name = "channelId") String channelId,
            @ApiParam(value = "超时时间,单位：毫秒", example = "10000") @RequestParam(name = "outTime", required = true) int outTime,
            @ApiParam(value = "请求参数JSON串") @RequestParam(name = "customBody", required = true) String customBody) {
        log.info("个人不良信息查询-简版请求--->channelId:{}--->outTime:{},--->customBody:{}", channelId, outTime, customBody);
        //日志记录通道类型 - 同步
        ServletRequestAttributes servletRequestAttributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        HttpServletRequest request = servletRequestAttributes.getRequest();
        //记录日志信息09
        recordQueryRequestLog(request);
        ChannelTypeHandleResponseObject responseObject = new ChannelTypeHandleResponseObject();
        try {
            responseObject = channelService.validateIllegalInfo(request, customBody, outTime, "1", ChannelConstant.ILLEGAL_INFO_SIMPLE);
            //日志记录通道响应码
            if (responseObject.getCode() != 1) {
                request.setAttribute(Constant.ACCESSLOG_MG_RESPONSE_CODE_KEY, Constant.CHANNEL_LOG_ERROR_CODE);
            } else {
                log.info("ifJk:{}", request.getHeader(Constant.CUSTOMER_RETURN_TYPE_PARA));
                if (Constant.CUSTOMER_RETURN_JK.equals(request.getHeader(Constant.CUSTOMER_RETURN_TYPE_PARA))) {

                    String code = supplierProperties.getCode().get(responseObject.getResultCode());
                    log.info("resultCode:{},jkCode:{}", responseObject.getResultCode(), code);
                    if (code != null) {
                        responseObject.setResultCode(Integer.valueOf(code));
                    }
                }
                request.setAttribute(Constant.ACCESSLOG_MG_RESPONSE_CODE_KEY, Constant.CHANNEL_LOG_SUCCESS_CODE);
            }
            //日志记录通道查询参数
            if (customBody != null) {
                try {
                    request.setAttribute(Constant.CHANNEL_LOG_QUERY, Base64.encodeBase64String(customBody.getBytes("utf-8")));
                } catch (UnsupportedEncodingException e) {
                    ExceptionUtils.handleThrowable(e);
                    log.error("异常信息：{}", e);
                }
            }

            request.setAttribute(Constant.ACCESSLOG_MG_ISCHARGE_KEY, responseObject.getIsCharge());
            return responseObject;
        } catch (Exception ex) {
            log.error("message:{}", ex);
            return responseObject;
        }
    }

    public void recordQueryRequestLog(HttpServletRequest request) {
        request.setAttribute(LOG_FEE_PARA, Constant.IS_NOT_CHARGE);
        request.setAttribute(LOG_ERROR_PARA, Constant.CHANNEL_LOG_ERROR_CODE);
        request.setAttribute(Constant.LOG_UPSTREAM_RESPONSE_CODE, Constant.LOG_UPSTREAM_DEFAULT_RESPONSE_CODE);
        request.setAttribute(LOG_REQ_TYPE, Constant.CHANNEL_TYPE_ASYNC);
        request.setAttribute(Constant.IFCJ_KEY, Constant.IFCJ_CJ);
        log.info("traceIdString:{},headerTraceId:{}", tracer.currentSpan().context().traceIdString(), request.getHeader(LOG_TREACEID));
        request.setAttribute(LOG_TREACEID, tracer.currentSpan().context().traceIdString());
        request.setAttribute(LOG_SPAN_ID, tracer.currentSpan().context().spanId());
        request.setAttribute(LOG_PARENT_SPAN_ID, tracer.currentSpan().context().parentId());
        request.setAttribute(LOG_REAL_IP, request.getHeader(LOG_REAL_IP));
        request.setAttribute(LOG_CHANNEL_ID, request.getHeader(LOG_CHANNEL_ID));
        request.setAttribute(LOG_PRICE, request.getHeader(LOG_PRICE));
        request.setAttribute(LOG_PRODUCT_ID, request.getHeader(LOG_PRODUCT_ID));
        request.setAttribute(LOG_USER_ID, request.getHeader(LOG_USER_ID));

    }

    @ApiOperation(value = "个人不良信息查询V2-简版", notes = "")
    @PostMapping(value = "/getIllegalInfoV2Simple.do")
    public ChannelTypeHandleResponseObject getIllegalInfoV2Simple(
            @ApiParam(value = "通道ID") @RequestHeader(name = "channelId") String channelId,
            @ApiParam(value = "超时时间,单位：毫秒", example = "10000") @RequestParam(name = "outTime", required = true) int outTime,
            @ApiParam(value = "请求参数JSON串") @RequestParam(name = "customBody", required = true) String customBody) {
        log.info("个人不良信息查询V2-简版请求--->channelId:{}--->outTime:{},--->customBody:{}", channelId, outTime, customBody);
        Long startTime = System.currentTimeMillis();
        //日志记录通道类型 - 同步
        ServletRequestAttributes servletRequestAttributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        HttpServletRequest request = servletRequestAttributes.getRequest();
        ChannelTypeHandleResponseObject responseObject = new ChannelTypeHandleResponseObject();
        try {
            responseObject = channelService.validateIllegalInfoV2(request, customBody, outTime,
                    ChannelConstant.ILLEGAL_INFO_SIMPLE, ChannelConstant.ILLEGAL_INFO_METHOD_NORMAL_RESPONSE);
            log.info("调用业务接口时延:{}", System.currentTimeMillis() - startTime);

            recodeIllegalInfoV2RequestLog(responseObject, request, customBody);
            if (System.currentTimeMillis() - startTime < delayedTime) {
                delay();
            }
            log.info("action总体耗时：{}", System.currentTimeMillis() - startTime);
            return responseObject;
        } catch (Exception ex) {
            log.error("message:{}", ex);
            return responseObject;
        }
    }

    @ApiOperation(value = "个人不良信息查询V2-评分版", notes = "")
    @PostMapping(value = "/getIllegalInfoV2Score.do")
    public ChannelTypeHandleResponseObject getIllegalInfoV2Score(
            @ApiParam(value = "通道ID") @RequestHeader(name = "channelId") String channelId,
            @ApiParam(value = "超时时间,单位：毫秒", example = "10000") @RequestParam(name = "outTime", required = true) int outTime,
            @ApiParam(value = "请求参数JSON串") @RequestParam(name = "customBody", required = true) String customBody) {
        log.info("个人不良信息查询V2-评分版请求--->channelId:{}--->outTime:{},--->customBody:{}", channelId, outTime, customBody);
        Long startTime = System.currentTimeMillis();
        //日志记录通道类型 - 同步
        ServletRequestAttributes servletRequestAttributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        HttpServletRequest request = servletRequestAttributes.getRequest();
        ChannelTypeHandleResponseObject responseObject = new ChannelTypeHandleResponseObject();
        try {
            responseObject = channelService.validateIllegalInfoV2(request, customBody, outTime,
                    ChannelConstant.ILLEGAL_INFO_SCORE, ChannelConstant.ILLEGAL_INFO_METHOD_NORMAL_RESPONSE);
            log.info("调用业务接口时延:{}", System.currentTimeMillis() - startTime);

            recodeIllegalInfoV2RequestLog(responseObject, request, customBody);
            if (System.currentTimeMillis() - startTime < delayedTime) {
                delay();
            }
            log.info("action总体耗时：{}", System.currentTimeMillis() - startTime);
            return responseObject;
        } catch (Exception ex) {
            log.error("message:{}", ex);
            return responseObject;
        }
    }

    @ApiOperation(value = "个人不良信息查询V2-详版", notes = "")
    @PostMapping(value = "/getIllegalInfoV2Result.do")
    public ChannelTypeHandleResponseObject getIllegalInfoV2Result(
            @ApiParam(value = "通道ID") @RequestHeader(name = "channelId") String channelId,
            @ApiParam(value = "超时时间,单位：毫秒", example = "10000") @RequestParam(name = "outTime", required = true) int outTime,
            @ApiParam(value = "请求参数JSON串") @RequestParam(name = "customBody", required = true) String customBody) {
        log.info("个人不良信息查询V2-详版版请求--->channelId:{}--->outTime:{},--->customBody:{}", channelId, outTime, customBody);
        Long startTime = System.currentTimeMillis();
        //日志记录通道类型 - 同步
        ServletRequestAttributes servletRequestAttributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        HttpServletRequest request = servletRequestAttributes.getRequest();
        ChannelTypeHandleResponseObject responseObject = new ChannelTypeHandleResponseObject();
        try {
            responseObject = channelService.validateIllegalInfoV2(request, customBody, outTime,
                    ChannelConstant.ILLEGAL_INFO_RESULT, ChannelConstant.ILLEGAL_INFO_METHOD_NORMAL_RESPONSE);
            log.info("调用业务接口时延:{}", System.currentTimeMillis() - startTime);

            recodeIllegalInfoV2RequestLog(responseObject, request, customBody);
            if (System.currentTimeMillis() - startTime < delayedTime) {
                delay();
            }
            log.info("action总体耗时：{}", System.currentTimeMillis() - startTime);
            return responseObject;
        } catch (Exception ex) {
            log.error("message:{}", ex);
            return responseObject;
        }
    }

    @ApiOperation(value = "个人不良信息查询V2-定制版", notes = "")
    @PostMapping(value = "/getIllegalInfoV2DrugEscape.do")
    public ChannelTypeHandleResponseObject getIllegalInfoV2DrugEscape(
            @ApiParam(value = "通道ID") @RequestHeader(name = "channelId") String channelId,
            @ApiParam(value = "超时时间,单位：毫秒", example = "10000") @RequestParam(name = "outTime", required = true) int outTime,
            @ApiParam(value = "请求参数JSON串") @RequestParam(name = "customBody", required = true) String customBody) {
        log.info("个人不良信息查询V2-定制版请求--->channelId:{}--->outTime:{},--->customBody:{}", channelId, outTime, customBody);
        Long startTime = System.currentTimeMillis();
        //日志记录通道类型 - 同步
        ServletRequestAttributes servletRequestAttributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        HttpServletRequest request = servletRequestAttributes.getRequest();
        ChannelTypeHandleResponseObject responseObject = new ChannelTypeHandleResponseObject();
        try {
            // scm解密
            String decodeCustomBody = channelService.getDecodeIdCard(request, customBody, outTime, ChannelConstant.DECODE_TYPE_MD5, responseObject);
            log.info("scm解密耗时：{}", System.currentTimeMillis() - startTime);

            if (StringUtils.isNotBlank(decodeCustomBody)) {
                customBody = decodeCustomBody;
                // 调用业务接口
                responseObject = channelService.validateIllegalInfoV2(request, customBody, outTime,
                        ChannelConstant.ILLEGAL_INFO_DRUG_ESCAPE, ChannelConstant.ILLEGAL_INFO_METHOD_NORMAL_RESPONSE);
            }
            log.info("调用业务接口时延:{}", System.currentTimeMillis() - startTime);

            recodeIllegalInfoV2RequestLog(responseObject, request, customBody);
            if (System.currentTimeMillis() - startTime < delayedTime) {
                delay();
            }
            log.info("action总体耗时：{}", System.currentTimeMillis() - startTime);
            return responseObject;
        } catch (Exception ex) {
            log.error("message:{}", ex);
            return responseObject;
        }
    }

    @ApiOperation(value = "个人不良信息查询V2-简版-SHA256", notes = "")
    @PostMapping(value = "/getIllegalInfoV2SimpleSHA256.do")
    public ChannelTypeHandleResponseObject getIllegalInfoV2SimpleSHA256(
            @ApiParam(value = "通道ID") @RequestHeader(name = "channelId") String channelId,
            @ApiParam(value = "超时时间,单位：毫秒", example = "10000") @RequestParam(name = "outTime", required = true) int outTime,
            @ApiParam(value = "请求参数JSON串") @RequestParam(name = "customBody", required = true) String customBody) {
        log.info("个人不良信息查询V2-简版请求--->channelId:{}--->outTime:{},--->customBody:{}", channelId, outTime, customBody);
        Long startTime = System.currentTimeMillis();
        //日志记录通道类型 - 同步
        ServletRequestAttributes servletRequestAttributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        HttpServletRequest request = servletRequestAttributes.getRequest();
        ChannelTypeHandleResponseObject responseObject = new ChannelTypeHandleResponseObject();
        try {
            // scm解密
            String decodeCustomBody = channelService.getDecodeIdCard(request, customBody, outTime, ChannelConstant.DECODE_TYPE_SHA256, responseObject);
            log.info("scm解密耗时：{}", System.currentTimeMillis() - startTime);

            if (StringUtils.isNotBlank(decodeCustomBody)) {
                customBody = decodeCustomBody;
                // 调用业务接口
                responseObject = channelService.validateIllegalInfoV2(request, customBody, outTime,
                        ChannelConstant.ILLEGAL_INFO_SIMPLE, ChannelConstant.ILLEGAL_INFO_METHOD_NORMAL_RESPONSE);
            }
            log.info("调用业务接口时延:{}", System.currentTimeMillis() - startTime);

            recodeIllegalInfoV2RequestLog(responseObject, request, customBody);
            if (System.currentTimeMillis() - startTime < delayedTime) {
                delay();
            }
            log.info("action总体耗时：{}", System.currentTimeMillis() - startTime);
            return responseObject;
        } catch (Exception ex) {
            log.error("message:{}", ex);
            return responseObject;
        }
    }

    @ApiOperation(value = "个人不良信息查询V2-评分版-SHA256", notes = "")
    @PostMapping(value = "/getIllegalInfoV2ScoreSHA256.do")
    public ChannelTypeHandleResponseObject getIllegalInfoV2ScoreSHA256(
            @ApiParam(value = "通道ID") @RequestHeader(name = "channelId") String channelId,
            @ApiParam(value = "超时时间,单位：毫秒", example = "10000") @RequestParam(name = "outTime", required = true) int outTime,
            @ApiParam(value = "请求参数JSON串") @RequestParam(name = "customBody", required = true) String customBody) {
        log.info("个人不良信息查询V2-评分版SHA256请求--->channelId:{}--->outTime:{},--->customBody:{}", channelId, outTime, customBody);
        Long startTime = System.currentTimeMillis();
        //日志记录通道类型 - 同步
        ServletRequestAttributes servletRequestAttributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        HttpServletRequest request = servletRequestAttributes.getRequest();
        ChannelTypeHandleResponseObject responseObject = new ChannelTypeHandleResponseObject();
        try {
            // 查看是否命中挡板数据
            boolean isIllegalInfoV2ScoreSHA256TestData = channelService.isIllegalInfoV2ScoreSHA256TestData(request, customBody, responseObject);
            String decodeCustomBody = "";
            if (!isIllegalInfoV2ScoreSHA256TestData) {
                // 未命中, 执行scm解密
                decodeCustomBody = channelService.getDecodeIdCard(request, customBody, outTime, ChannelConstant.DECODE_TYPE_SHA256, responseObject);
            }

            log.info("scm解密耗时：{}", System.currentTimeMillis() - startTime);

            if (StringUtils.isNotBlank(decodeCustomBody)) {
                customBody = decodeCustomBody;
                // 调用业务接口
                responseObject = channelService.validateIllegalInfoV2(request, customBody, outTime,
                        ChannelConstant.ILLEGAL_INFO_SCORE, ChannelConstant.ILLEGAL_INFO_METHOD_NORMAL_RESPONSE);
            }
            log.info("调用业务接口时延:{}", System.currentTimeMillis() - startTime);

            recodeIllegalInfoV2RequestLog(responseObject, request, customBody);
            if (System.currentTimeMillis() - startTime < delayedTime) {
                delay();
            }
            log.info("action总体耗时：{}", System.currentTimeMillis() - startTime);
            return responseObject;
        } catch (Exception ex) {
            log.error("message:{}", ex);
            return responseObject;
        }
    }

    @ApiOperation(value = "个人不良信息查询V2-评分版-MD5", notes = "")
    @PostMapping(value = "/getIllegalInfoV2ScoreMD5.do")
    public ChannelTypeHandleResponseObject getIllegalInfoV2ScoreMD5(
            @ApiParam(value = "通道ID") @RequestHeader(name = "channelId") String channelId,
            @ApiParam(value = "超时时间,单位：毫秒", example = "10000") @RequestParam(name = "outTime", required = true) int outTime,
            @ApiParam(value = "请求参数JSON串") @RequestParam(name = "customBody", required = true) String customBody) {
        log.info("个人不良信息查询V2-评分版SHA256请求--->channelId:{}--->outTime:{},--->customBody:{}", channelId, outTime, customBody);
        Long startTime = System.currentTimeMillis();
        //日志记录通道类型 - 同步
        ServletRequestAttributes servletRequestAttributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        HttpServletRequest request = servletRequestAttributes.getRequest();
        ChannelTypeHandleResponseObject responseObject = new ChannelTypeHandleResponseObject();
        try {
            // scm解密
            String decodeCustomBody = channelService.getDecodeIdCard(request, customBody, outTime, ChannelConstant.DECODE_TYPE_MD5, responseObject);
            log.info("scm解密耗时：{}", System.currentTimeMillis() - startTime);

            if (StringUtils.isNotBlank(decodeCustomBody)) {
                customBody = decodeCustomBody;
                // 调用业务接口
                responseObject = channelService.validateIllegalInfoV2(request, customBody, outTime,
                        ChannelConstant.ILLEGAL_INFO_SCORE, ChannelConstant.ILLEGAL_INFO_METHOD_NORMAL_RESPONSE);
            }
            log.info("调用业务接口时延:{}", System.currentTimeMillis() - startTime);

            recodeIllegalInfoV2RequestLog(responseObject, request, customBody);
            if (System.currentTimeMillis() - startTime < delayedTime) {
                delay();
            }
            log.info("action总体耗时：{}", System.currentTimeMillis() - startTime);
            return responseObject;
        } catch (Exception ex) {
            log.error("message:{}", ex);
            return responseObject;
        }
    }

    @ApiOperation(value = "个人不良信息查询V2-定制版-AES", notes = "")
    @PostMapping(value = "/getIllegalInfoV2DrugEscapeAES.do")
    public ChannelTypeHandleResponseObject getIllegalInfoV2DrugEscapeAES(
            @ApiParam(value = "通道ID") @RequestHeader(name = "channelId") String channelId,
            @ApiParam(value = "超时时间,单位：毫秒", example = "10000") @RequestParam(name = "outTime", required = true) int outTime,
            @ApiParam(value = "请求参数JSON串") @RequestBody String customBody) {
        log.info("个人不良信息查询V2-定制版请求--->channelId:{}--->outTime:{},--->customBody:{}", channelId, outTime, customBody);
        Long startTime = System.currentTimeMillis();
        //日志记录通道类型 - 同步
        ServletRequestAttributes servletRequestAttributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        HttpServletRequest request = servletRequestAttributes.getRequest();
        ChannelTypeHandleResponseObject responseObject = new ChannelTypeHandleResponseObject();
        try {
            // aes解密
            String decodeCustomBody = channelService.getAesDecodeIdCard(request, customBody, responseObject);
            log.info("aes解密耗时：{}", System.currentTimeMillis() - startTime);

            if (StringUtils.isNotBlank(decodeCustomBody)) {
                customBody = decodeCustomBody;
                // 调用业务接口
                responseObject = channelService.validateIllegalInfoV2(request, customBody, outTime,
                        ChannelConstant.ILLEGAL_INFO_DRUG_ESCAPE, ChannelConstant.ILLEGAL_INFO_METHOD_NORMAL_RESPONSE);
            }
            log.info("调用业务接口时延:{}", System.currentTimeMillis() - startTime);

            recodeIllegalInfoV2RequestLog(responseObject, request, customBody);
            if (System.currentTimeMillis() - startTime < delayedTime) {
                delay();
            }
            log.info("action总体耗时：{}", System.currentTimeMillis() - startTime);
            return responseObject;
        } catch (Exception ex) {
            log.error("message:{}", ex);
            return responseObject;
        }
    }

    @ApiOperation(value = "个人不良信息查询V2-定制版-AES-MD5", notes = "")
    @PostMapping(value = "/getIllegalInfoV2DrugEscapeAESMD5.do")
    public ChannelTypeHandleResponseObject getIllegalInfoV2DrugEscapeAESMD5(
            @ApiParam(value = "通道ID") @RequestHeader(name = "channelId") String channelId,
            @ApiParam(value = "超时时间,单位：毫秒", example = "10000") @RequestParam(name = "outTime", required = true) int outTime,
            @ApiParam(value = "请求参数JSON串") @RequestBody String customBody) {
        log.info("个人不良信息查询V2-定制版请求--->channelId:{}--->outTime:{},--->customBody:{}", channelId, outTime, customBody);
        Long startTime = System.currentTimeMillis();
        //日志记录通道类型 - 同步
        ServletRequestAttributes servletRequestAttributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        HttpServletRequest request = servletRequestAttributes.getRequest();
        ChannelTypeHandleResponseObject responseObject = new ChannelTypeHandleResponseObject();
        try {
            // aes解密
            String decodeCustomBody = channelService.getAesMd5DecodeIdCode(request, customBody, responseObject, outTime);
            log.info("aes+md5解密耗时：{}", System.currentTimeMillis() - startTime);

            if (StringUtils.isNotBlank(decodeCustomBody)) {
                customBody = decodeCustomBody;
                // 调用业务接口
                responseObject = channelService.validateIllegalInfoV2(request, customBody, outTime,
                        ChannelConstant.ILLEGAL_INFO_DRUG_ESCAPE, ChannelConstant.ILLEGAL_INFO_METHOD_NORMAL_RESPONSE);
            }
            log.info("调用业务接口时延:{}", System.currentTimeMillis() - startTime);

            recodeIllegalInfoV2RequestLog(responseObject, request, customBody);
            if (System.currentTimeMillis() - startTime < delayedTime) {
                delay();
            }
            log.info("action总体耗时：{}", System.currentTimeMillis() - startTime);
            return responseObject;
        } catch (Exception ex) {
            log.error("message:{}", ex);
            return responseObject;
        }
    }

    @ApiOperation(value = "个人不良信息查询V2-简版-身份证SHA256-姓名明文", notes = "")
    @PostMapping(value = "/getIllegalInfoV2SimpleSHA256IdCode.do")
    public ChannelTypeHandleResponseObject getIllegalInfoV2SimpleSHA256IdCode(
            @ApiParam(value = "通道ID") @RequestHeader(name = "channelId") String channelId,
            @ApiParam(value = "超时时间,单位：毫秒", example = "10000") @RequestParam(name = "outTime", required = true) int outTime,
            @ApiParam(value = "请求参数JSON串") @RequestParam(name = "customBody", required = true) String customBody) {
        log.info("个人不良信息查询V2-简版-身份证SHA256-姓名明文--->channelId:{}--->outTime:{},--->customBody:{}", channelId, outTime, customBody);
        Long startTime = System.currentTimeMillis();
        //日志记录通道类型 - 同步
        ServletRequestAttributes servletRequestAttributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        HttpServletRequest request = servletRequestAttributes.getRequest();
        ChannelTypeHandleResponseObject responseObject = new ChannelTypeHandleResponseObject();
        try {
            // scm解密
            String decodeCustomBody = channelService.getDecodeIdCode(request, customBody, outTime, ChannelConstant.DECODE_TYPE_SHA256, responseObject);
            log.info("scm解密耗时：{}", System.currentTimeMillis() - startTime);

            if (StringUtils.isNotBlank(decodeCustomBody)) {
                customBody = decodeCustomBody;
                // 调用业务接口
                responseObject = channelService.validateIllegalInfoV2(request, customBody, outTime,
                        ChannelConstant.ILLEGAL_INFO_SIMPLE, ChannelConstant.ILLEGAL_INFO_METHOD_NORMAL_RESPONSE);
            }
            log.info("调用业务接口时延:{}", System.currentTimeMillis() - startTime);

            recodeIllegalInfoV2RequestLog(responseObject, request, customBody);
            if (System.currentTimeMillis() - startTime < delayedTime) {
                delay();
            }
            log.info("action总体耗时：{}", System.currentTimeMillis() - startTime);
            return responseObject;
        } catch (Exception ex) {
            log.error("message:{}", ex);
            return responseObject;
        }
    }

    @ApiOperation(value = "个人不良信息查询V2-评分版-SM3", notes = "")
    @PostMapping(value = "/getIllegalInfoV2ScoreSM3.do")
    public ChannelTypeHandleResponseObject getIllegalInfoV2ScoreSM3(
            @ApiParam(value = "通道ID") @RequestHeader(name = "channelId") String channelId,
            @ApiParam(value = "超时时间,单位：毫秒", example = "10000") @RequestParam(name = "outTime", required = true) int outTime,
            @ApiParam(value = "请求参数JSON串") @RequestParam(name = "customBody", required = true) String customBody) {
        log.info("个人不良信息查询V2-评分版SM3请求--->channelId:{}--->outTime:{},--->customBody:{}", channelId, outTime, customBody);
        Long startTime = System.currentTimeMillis();
        //日志记录通道类型 - 同步
        ServletRequestAttributes servletRequestAttributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        HttpServletRequest request = servletRequestAttributes.getRequest();
        ChannelTypeHandleResponseObject responseObject = new ChannelTypeHandleResponseObject();
        try {
            // scm解密
            String decodeCustomBody = channelService.getDecodeIdCard(request, customBody, outTime, ChannelConstant.DECODE_TYPE_SM3, responseObject);
            log.info("scm解密耗时：{}", System.currentTimeMillis() - startTime);

            if (StringUtils.isNotBlank(decodeCustomBody)) {
                customBody = decodeCustomBody;
                // 调用业务接口
                responseObject = channelService.validateIllegalInfoV2(request, customBody, outTime,
                        ChannelConstant.ILLEGAL_INFO_SCORE, ChannelConstant.ILLEGAL_INFO_METHOD_NORMAL_RESPONSE);
            }
            log.info("调用业务接口时延:{}", System.currentTimeMillis() - startTime);

            recodeIllegalInfoV2RequestLog(responseObject, request, customBody);
            if (System.currentTimeMillis() - startTime < delayedTime) {
                delay();
            }
            log.info("action总体耗时：{}", System.currentTimeMillis() - startTime);
            return responseObject;
        } catch (Exception ex) {
            log.error("message:{}", ex);
            return responseObject;
        }
    }

    @ApiOperation(value = "个人不良信息查询V2-评分版-SM3-拼接版", notes = "")
    @PostMapping(value = "/getIllegalInfoV2ScoreSM3Criminal.do")
    public ChannelTypeHandleResponseObject getIllegalInfoV2ScoreSM3Criminal(
            @ApiParam(value = "通道ID") @RequestHeader(name = "channelId") String channelId,
            @ApiParam(value = "超时时间,单位：毫秒", example = "10000") @RequestParam(name = "outTime", required = true) int outTime,
            @ApiParam(value = "请求参数JSON串") @RequestParam(name = "customBody", required = true) String customBody) {
        log.info("个人不良信息查询V2-评分版SM3请求--->channelId:{}--->outTime:{},--->customBody:{}", channelId, outTime, customBody);
        Long startTime = System.currentTimeMillis();
        //日志记录通道类型 - 同步
        ServletRequestAttributes servletRequestAttributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        HttpServletRequest request = servletRequestAttributes.getRequest();
        ChannelTypeHandleResponseObject responseObject = new ChannelTypeHandleResponseObject();
        try {
            // scm解密
            String decodeCustomBody = channelService.getDecodeIdCard(request, customBody, outTime, ChannelConstant.DECODE_TYPE_SM3, responseObject);
            log.info("scm解密耗时：{}", System.currentTimeMillis() - startTime);

            if (StringUtils.isNotBlank(decodeCustomBody)) {
                customBody = decodeCustomBody;
                // 调用业务接口
                responseObject = channelService.validateIllegalInfoV2(request, customBody, outTime,
                        ChannelConstant.ILLEGAL_INFO_SCORE, ChannelConstant.ILLEGAL_INFO_METHOD_SPECIAL);
            }
            log.info("调用业务接口时延:{}", System.currentTimeMillis() - startTime);

            recodeIllegalInfoV2RequestLog(responseObject, request, customBody);
            if (System.currentTimeMillis() - startTime < delayedTime) {
                delay();
            }
            log.info("action总体耗时：{}", System.currentTimeMillis() - startTime);
            return responseObject;
        } catch (Exception ex) {
            log.error("message:{}", ex);
            return responseObject;
        }
    }



    private void delay() {
        Long startTime = System.currentTimeMillis();
        try {
            Future<String> delayTask = asyncDelayTask.doDelay();
            delayTask.get(delayedTime*2, TimeUnit.MILLISECONDS);
        } catch (InterruptedException ie) {
            log.info("InterruptedException, 强制结束子线程,子线程耗时：{}, {}", System.currentTimeMillis() - startTime, ie);
            ErrorUtils.captureException(ie);
        } catch (ExecutionException ee) {
            log.info("ExecutionException, 强制结束子线程,子线程耗时：{}, {}", System.currentTimeMillis() - startTime, ee);
            ErrorUtils.captureException(ee);
        } catch (TimeoutException te) {
            log.info("TimeoutException, 强制结束子线程,子线程耗时：{}, {}", System.currentTimeMillis() - startTime, te);
            ErrorUtils.captureException(te);
        }
    }

    private void recodeIllegalInfoV2RequestLog(ChannelTypeHandleResponseObject responseObject, HttpServletRequest request, String customBody) {
        //日志记录通道响应码
        if (responseObject.getCode() != 1) {
            request.setAttribute(Constant.ACCESSLOG_MG_RESPONSE_CODE_KEY, Constant.CHANNEL_LOG_ERROR_CODE);
        } else {
            String ifJkCode = request.getHeader(Constant.CUSTOMER_RETURN_TYPE_PARA);
            log.info("ifJk:{}", ifJkCode);
            if (Constant.CUSTOMER_RETURN_JK.equals(ifJkCode)) {

                String code = supplierProperties.getCode().get(responseObject.getResultCode());
                log.info("resultCode:{},jkCode:{}", responseObject.getResultCode(), code);
                if (code != null) {
                    responseObject.setResultCode(Integer.valueOf(code));
                }
            }
            request.setAttribute(Constant.ACCESSLOG_MG_RESPONSE_CODE_KEY, Constant.CHANNEL_LOG_SUCCESS_CODE);
        }
        //日志记录通道查询参数
        if (customBody != null) {
            try {
                request.setAttribute(Constant.CHANNEL_LOG_QUERY, Base64.encodeBase64String(customBody.getBytes("utf-8")));
            } catch (UnsupportedEncodingException e) {
                ExceptionUtils.handleThrowable(e);
                log.error("异常信息：{}", e);
            }
        }

        request.setAttribute(Constant.ACCESSLOG_MG_ISCHARGE_KEY, responseObject.getIsCharge());

        request.setAttribute(LOG_FEE_PARA, responseObject.getIsCharge());
        request.setAttribute(LOG_TREACEID, tracer.currentSpan().context().traceIdString());
        request.setAttribute(LOG_REAL_IP, request.getHeader(LOG_REAL_IP));
        request.setAttribute(LOG_CHANNEL_ID, request.getHeader(LOG_CHANNEL_ID));
        request.setAttribute(LOG_PRICE, request.getHeader(LOG_PRICE));
        request.setAttribute(LOG_PRODUCT_ID, request.getHeader(LOG_PRODUCT_ID));
        request.setAttribute(LOG_USER_ID, request.getHeader(LOG_USER_ID));
        request.setAttribute(Constant.IFCJ_KEY, Constant.IFCJ_CJ);
        request.setAttribute(LOG_REQ_TYPE, Constant.CHANNEL_TYPE_SYNC);
    }

    @ApiOperation(value = "车辆信息查询", notes = "")
    @PostMapping(value = "/validateVehicleInfo.do")
    public ChannelTypeHandleResponseObject validateVehicleInfo(
            @ApiParam(value = "通道ID") @RequestHeader(name = "channelId") String channelId,
            @ApiParam(value = "超时时间,单位：毫秒", example = "10000") @RequestParam(name = "outTime", required = true) int outTime,
            @ApiParam(value = "请求参数JSON串") @RequestParam(name = "customBody", required = true) String customBody) {
        log.info("车辆信息查询--->channelId:{}--->outTime:{},--->customBody:{}", channelId, outTime, customBody);
        //日志记录通道类型 - 同步
        ServletRequestAttributes servletRequestAttributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        HttpServletRequest request = servletRequestAttributes.getRequest();
        ChannelTypeHandleResponseObject responseObject = new ChannelTypeHandleResponseObject();
        try {
            // 调用业务接口
            responseObject = channelService.validateVehicleInfo(request, customBody, outTime);
            recodeIllegalInfoV2RequestLog(responseObject, request, customBody);
            return responseObject;
        } catch (Exception ex) {
            log.error("message:{}", ex);
            return responseObject;
        }
    }
}
