package info.aspirecn.iov.sjjh.supplier10000040.task;

import com.alibaba.fastjson.JSON;
import info.aspirecn.iov.sjjh.supplier10000040.constant.ChannelConstant;
import info.aspirecn.iov.sjjh.supplier10000040.model.IllegalInfoParamExcelObject;
import info.aspirecn.iov.sjjh.supplier10000040.model.RedisParamObject;
import info.aspirecn.iov.sjjh.supplier10000040.service.ChannelService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * @description:
 * @author: xusonglin
 * @create: 2020/4/23 13:44
 * @version: V1.0
 **/
@Component
@Slf4j
public class IllegalInfoTask {
    @Autowired
    ChannelService channelService;

    @Autowired
    private StringRedisTemplate redisTemplate;

    @Value("${localDir.download}")
    private String localDownloadDir;

//    @Scheduled(cron = "0/10 * * * * ?")
//    public void illegalInfoTask() {
//        Set<String> illegalInfoRedisSetKey = redisTemplate.boundZSetOps("asynchronous-zSet-illegal-info").range(0, System.currentTimeMillis());
//        for (String redisNowTimeKey : illegalInfoRedisSetKey) {
//            String rankKey = "asynchronous-illegal-info-rank-" + redisNowTimeKey;
//            String redisSetKey = "asynchronous-illegal-info-" + redisNowTimeKey;
//            if (!redisTemplate.boundValueOps(rankKey).setIfAbsent("1")) {
//                log.info("当前有正在执行key为:{}的定时任务", "asynchronous-illegal-info-rank-" + redisNowTimeKey);
//                continue;
//            }
//            Set<String> set = redisTemplate.opsForSet().members(redisSetKey);
//            if (!set.isEmpty()) {
//                redisTemplate.boundValueOps(rankKey).expire(set.size(), TimeUnit.SECONDS);
//                boolean sendResult = channelService.sendRequest(set, redisSetKey);
//                if (sendResult) {
//                    boolean deleteKey = redisTemplate.delete(redisSetKey);
//                    log.info("key:{}，是否存在{}", redisSetKey, redisTemplate.hasKey(redisSetKey));
//                    log.info("删除key:{},结果:{}", redisSetKey, deleteKey);
//                    long deleteKeyFromZSet = redisTemplate.boundZSetOps("asynchronous-zSet-illegal-info").remove(redisNowTimeKey);
//                    log.info("移除key：{},中的列表值：{},结果:{}", "asynchronous-zSet-illegal-info", redisNowTimeKey, deleteKeyFromZSet);
//                }
//            } else {
//                continue;
//            }
//            boolean deleteRankKey = redisTemplate.delete(rankKey);
//            log.info("删除rankKey:{},结果:{}", rankKey, deleteRankKey);
//        }
//    }

    @Scheduled(cron = "0/10 * * * * ?")
    public void illegalInfoTask() {
        // 取出redis参数
        Map<Object, Object> illegalInfoRequestParams = redisTemplate.opsForHash().entries("asynchronous-illegal-info-request");
        if (illegalInfoRequestParams.size() == 0) {
            return;
        }
        List<IllegalInfoParamExcelObject> excelObjectList = new ArrayList<>();
        // 遍历所有参数
        for (Map.Entry<Object, Object> entry : illegalInfoRequestParams.entrySet()) {
            // 解析参数String
            RedisParamObject redisParamObject = JSON.toJavaObject(JSON.parseObject(entry.getValue().toString()), RedisParamObject.class);
            // 判断参数的状态是否是待处理
            if (redisParamObject.getStatus().equals(ChannelConstant.REQUEST_STATUS_1)) {
                // 从redis中重新取该条参数String,进行版本号比对，如果版本号一致，则修改状态
                String nowTimeParam = (String) redisTemplate.boundHashOps("asynchronous-illegal-info-request").get(redisParamObject.getTraceId());
                RedisParamObject nowTimeParamObject = JSON.toJavaObject(JSON.parseObject(nowTimeParam), RedisParamObject.class);
                if (nowTimeParamObject.getVersion().equals(redisParamObject.getVersion())) {
                    // 该条参数与redis中的版本号一致，进行修改状态操作，并且版本号+1
                    redisParamObject.setStatus(ChannelConstant.REQUEST_STATUS_2);
                    redisParamObject.setVersion(redisParamObject.getVersion() + 1);

                    redisTemplate.boundHashOps("asynchronous-illegal-info-request")
                            .put(redisParamObject.getTraceId(), JSON.toJSONString(redisParamObject));
                    log.info("illegalInfoTask -- 修改数据流水号：{}，修改为状态：2（处理中）", redisParamObject.getTraceId());
                    IllegalInfoParamExcelObject excelObject = new IllegalInfoParamExcelObject();
                    excelObject.setName(redisParamObject.getName());
                    excelObject.setIdCode(redisParamObject.getIdCode());
                    excelObject.setTraceId(redisParamObject.getInterfaceType() + "-" + redisParamObject.getTraceId());
                    excelObjectList.add(excelObject);
                }
            }
        }

        if (excelObjectList.size() == 0) {
            return;
        }
        boolean sendResult = channelService.sendRequest(excelObjectList);

        // 发送上游成功，则修改数据状态
        if (sendResult) {
            for (IllegalInfoParamExcelObject infoParamExcelObject : excelObjectList) {
                // 解析参数String
                String nowTimeParam = (String) redisTemplate.boundHashOps("asynchronous-illegal-info-request").get(infoParamExcelObject.getTraceId().split("-")[1]);
                RedisParamObject redisParamObject = JSON.toJavaObject(JSON.parseObject(nowTimeParam), RedisParamObject.class);
                redisParamObject.setStatus(ChannelConstant.REQUEST_STATUS_3);
                redisParamObject.setVersion(redisParamObject.getVersion() + 1);

                redisTemplate.boundHashOps("asynchronous-illegal-info-request")
                        .put(redisParamObject.getTraceId(), JSON.toJSONString(redisParamObject));
                log.info("illegalInfoTask -- 修改数据流水号：{}，修改为状态：3（已发送上游）", redisParamObject.getTraceId());
            }
        }
    }

    @Scheduled(cron = "0 0/30 * * * ?", zone = "Asia/Shanghai")
    public void dealParamInfo() {
        log.info("dealParamInfo -- 开始处理无结果数据");
        // 取出redis参数
        Map<Object, Object> illegalInfoRequestParams = redisTemplate.opsForHash().entries("asynchronous-illegal-info-request");
        Long nowTimestamp = System.currentTimeMillis();
        // 遍历所有参数
        for (Map.Entry<Object, Object> entry : illegalInfoRequestParams.entrySet()) {
            // 解析参数String
            RedisParamObject redisParamObject = JSON.toJavaObject(JSON.parseObject(entry.getValue().toString()), RedisParamObject.class);
            String traceId = redisParamObject.getTraceId();
            if (redisParamObject.getStatus().equals(ChannelConstant.REQUEST_STATUS_4)) {
                redisTemplate.boundHashOps("asynchronous-illegal-info-request").delete(traceId);
                log.info("dealParamInfo -- 移除处理完成的参数：{}", traceId);
                continue;
            }

            if ((redisParamObject.getStatus().equals(ChannelConstant.REQUEST_STATUS_2)
                    || redisParamObject.getStatus().equals(ChannelConstant.REQUEST_STATUS_3))
                    && (nowTimestamp - redisParamObject.getTimestamp() > 3600000)) {
                channelService.sendFailedMessage(traceId);
            }
        }
    }

    @Scheduled(cron = "0 0/5 * * * ?", zone = "Asia/Shanghai")
    public void removeFiles() {
        File camelFile = new File(localDownloadDir + ".camel" + File.separator);
        log.info("读取临时文件夹", localDownloadDir + ".camel" + File.separator);
        File[] files = camelFile.listFiles();

        for (File file : files) {
            boolean deleteResult = file.delete();
            log.info("删除文件：{}, 删除结果:{}", file.getName(), deleteResult);
        }
    }
}
