package info.aspirecn.iov.sjjh.supplier10000040.util;

import info.aspirecn.iov.sjjh.supplier10000040.model.IllegalInfoFtpResult;
import info.aspirecn.iov.sjjh.supplier10000040.model.IllegalInfoParamExcelObject;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.hssf.usermodel.HSSFCell;
import org.apache.poi.hssf.usermodel.HSSFRow;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.xssf.usermodel.XSSFCell;
import org.apache.poi.xssf.usermodel.XSSFRow;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;

import java.io.*;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

import static javax.xml.bind.JAXBIntrospector.getValue;

/**
 * @description:
 * @author: xusonglin
 * @create: 2020/4/22 15:00
 * @version: V1.0
 **/
@Slf4j
public class ExcelUtil {


    /**
     * suffix of excel 2003
     */
    public static final String OFFICE_EXCEL_V2003_SUFFIX = "xls";
    /**
     * suffix of excel 2007
     */
    public static final String OFFICE_EXCEL_V2007_SUFFIX = "xlsx";
    /**
     * suffix of excel 2010
     */
    public static final String OFFICE_EXCEL_V2010_SUFFIX = "xlsx";

    public static final String NOT_EXCEL_FILE = " is Not a Excel file!";

    public static final String EMPTY = "";
    public static final String DOT = ".";

    public static String NO_DEFINE = "no_define";//未定义的字段
    public static String DEFAULT_DATE_PATTERN = "yyyy年MM月dd日";//默认日期格式
    public static int DEFAULT_COLOUMN_WIDTH = 17;

    public static List<IllegalInfoFtpResult> readExcel(String path) {
        if (StringUtils.isBlank(path)) {
            throw new IllegalArgumentException(path + " excel file path is either null or empty");
        } else {
            String suffiex = getSuffiex(path);
            if (StringUtils.isBlank(suffiex)) {
                throw new IllegalArgumentException(path + " suffiex is either null or empty");
            }
            if (OFFICE_EXCEL_V2003_SUFFIX.equals(suffiex)) {
                return readXls(path);
            } else if (OFFICE_EXCEL_V2007_SUFFIX.equals(suffiex)) {
                return readXlsx(path);
            } else if (OFFICE_EXCEL_V2010_SUFFIX.equals(suffiex)) {
                return readXlsx(path);
            } else {
                throw new IllegalArgumentException(path + NOT_EXCEL_FILE);
            }
        }
    }

    public static List<IllegalInfoFtpResult> readXls(String path) {

        InputStream is = null;
        HSSFWorkbook hssfWorkbook = null;
        List<IllegalInfoFtpResult> list = new ArrayList<IllegalInfoFtpResult>();
        try {
            is = new FileInputStream(path);
            hssfWorkbook = new HSSFWorkbook(is);
            IllegalInfoFtpResult manFtpResult = null;

            // Read the Sheet
            for (int numSheet = 0; numSheet < hssfWorkbook.getNumberOfSheets(); numSheet++) {
                HSSFSheet hssfSheet = hssfWorkbook.getSheetAt(numSheet);
                if (hssfSheet == null) {
                    continue;
                }
                // Read the Row
                for (int rowNum = 0; rowNum <= hssfSheet.getLastRowNum(); rowNum++) {
                    HSSFRow hssfRow = hssfSheet.getRow(rowNum);
                    if (hssfRow != null) {
                        manFtpResult = new IllegalInfoFtpResult();
                        HSSFCell traceId = hssfRow.getCell(0);
                        HSSFCell name = hssfRow.getCell(1);
                        HSSFCell idCard = hssfRow.getCell(2);
                        HSSFCell isIdCard = hssfRow.getCell(3);
                        HSSFCell isEscape = hssfRow.getCell(4);
                        HSSFCell isPedigree = hssfRow.getCell(5);
                        HSSFCell isDrugs = hssfRow.getCell(6);
                        manFtpResult.setTraceId(getValue(traceId).toString());
                        manFtpResult.setName(getValue(name).toString());
                        manFtpResult.setIdCard(getValue(idCard).toString());
                        manFtpResult.setIsIdCard(new BigDecimal(getValue(isIdCard).toString()).intValue());
                        manFtpResult.setIsEscape(new BigDecimal(getValue(isEscape).toString()).intValue());
                        manFtpResult.setIsPedigree(new BigDecimal(getValue(isPedigree).toString()).intValue());
                        manFtpResult.setIsDrugs(new BigDecimal(getValue(isDrugs).toString()).intValue());
                        list.add(manFtpResult);
                    }
                }
            }
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        } finally {

            if (hssfWorkbook != null) {
                try {
                    hssfWorkbook.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }

            if (is != null) {
                try {
                    is.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return list;
    }

    public static List<IllegalInfoFtpResult> readXlsx(String path) {
        InputStream is = null;
        XSSFWorkbook xssfWorkbook = null;
        IllegalInfoFtpResult manFtpResult = null;
        List<IllegalInfoFtpResult> list = new ArrayList<>();

        try {

            is = new FileInputStream(path);
            xssfWorkbook = new XSSFWorkbook(is);

            // Read the Sheet
            for (int numSheet = 0; numSheet < xssfWorkbook.getNumberOfSheets(); numSheet++) {
                XSSFSheet xssfSheet = xssfWorkbook.getSheetAt(numSheet);
                if (xssfSheet == null) {
                    continue;
                }

                // Read the Row
                for (int rowNum = 0; rowNum <= xssfSheet.getLastRowNum(); rowNum++) {
                    XSSFRow xssfRow = xssfSheet.getRow(rowNum);
                    if (xssfRow != null) {
                        manFtpResult = new IllegalInfoFtpResult();
                        XSSFCell traceId = xssfRow.getCell(0);
                        XSSFCell name = xssfRow.getCell(1);
                        XSSFCell idCard = xssfRow.getCell(2);
                        XSSFCell isIdCard = xssfRow.getCell(3);
                        XSSFCell isEscape = xssfRow.getCell(4);
                        XSSFCell isPedigree = xssfRow.getCell(5);
                        XSSFCell isDrugs = xssfRow.getCell(6);
                        manFtpResult.setTraceId(getValue(traceId).toString());
                        manFtpResult.setName(getValue(name).toString());
                        manFtpResult.setIdCard(getValue(idCard).toString());
                        manFtpResult.setIsIdCard(new BigDecimal(getValue(isIdCard).toString()).intValue());
                        manFtpResult.setIsEscape(new BigDecimal(getValue(isEscape).toString()).intValue());
                        manFtpResult.setIsPedigree(new BigDecimal(getValue(isPedigree).toString()).intValue());
                        manFtpResult.setIsDrugs(new BigDecimal(getValue(isDrugs).toString()).intValue());
                        list.add(manFtpResult);
                    }
                }
            }

        } catch (IOException e) {
            e.printStackTrace();
        } finally {

            if (xssfWorkbook != null) {
                try {
                    xssfWorkbook.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }

            if (is != null) {
                try {
                    is.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }

        return list;
    }

    public static String getSuffiex(String path) {
        if (StringUtils.isBlank(path)) {
            return EMPTY;
        }
        int index = path.lastIndexOf(DOT);
        if (index == -1) {
            return EMPTY;
        }
        return path.substring(index + 1, path.length());
    }

    public static String  createExcel(OutputStream outputStream, List<IllegalInfoParamExcelObject> list) throws IOException {

        XSSFWorkbook wb = new XSSFWorkbook();
        //建立新的sheet对象（excel的表单）
        XSSFSheet sheet=wb.createSheet();
        for (int i = 0; i < list.size(); i++) {
            XSSFRow rowx=sheet.createRow(i);
            rowx.createCell(0).setCellValue(list.get(i).getTraceId());
            rowx.createCell(1).setCellValue(list.get(i).getName());
            rowx.createCell(2).setCellValue(list.get(i).getIdCode());
        }
        //输出Excel文件
        wb.write(outputStream);
        wb.close();
        outputStream.close();
        return null;
    }
}
